# zib -- zzznowman's IRC bot
# Copyright (C) 2006  Fabian Pietsch <fabian@zzznowman.dyndns.org>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Request object

package ZIB::Request;
use strict;
use warnings;
use vars qw($VERSION);
use Carp;

$VERSION = 20060522.1;

my %types;
my @emptyreplies = ('[empty reply]');

sub new($$$$@)
{
	my ($class, $how, $who, $what, @args) = @_;

	die("invalid request type $what, died")
	  unless (defined($types{$what}));

	my $self = {
		via => $how,
		who => $who,
		type => $what,
		arguments => \@args,
		reply => [],
		ready => 0
	};

	return bless($self, $class);
}

sub via($)   { return $_[0]->{via} }
sub who($)   { return $_[0]->{who} }
sub type($)  { return $_[0]->{type} }
sub args($)  { return (wantarray ? @{$_[0]->{arguments}} : $_[0]->{arguments}) }
sub reply($) { return (wantarray ? @{$_[0]->{reply}}     : $_[0]->{reply}) }

sub ready($;$) {
	my ($self, $newval) = @_;

	if (scalar(@_) > 1)
	{
		$self->{ready} = $newval;
	}

	return $self->{ready};
}

sub queue_reply($$)
{
	my ($self, $line) = @_;

	push(@{$self->reply()}, $line);
}

sub do_request($)
{
	my ($self) = @_;

	return if ($self->ready());
	$types{$self->type()}->($self);
}

sub register_types($&@)
{
	my ($class, $coderef, @typenames) = @_;

	croak("ZIB::Request: can't register types: $coderef is not a CODE ref, died")
	  unless (ref($coderef) eq 'CODE');

	foreach my $typename (@typenames)
	{
		$types{$typename} = $coderef;
	}
}

sub unregister_types($@)
{
	my ($class, @typenames) = @_;

	foreach my $typename (@typenames)
	{
		delete($types{$typename});
	}
}

sub unregister_all_types($)
{
	%types = ();
}

sub emptyreplies($;@)
{
	my ($class, @newreplies) = @_;

	if (scalar(@_) > 1)
	{
		@emptyreplies = @newreplies;
	}

	return @emptyreplies;
}

sub add_emptyreplies($@)
{
	my ($class, @newreplies) = @_;

	push(@emptyreplies, @newreplies);
	return @emptyreplies;
}

1;
