# zib -- zzznowman's IRC bot
# Copyright (C) 2006  Fabian Pietsch <fabian@zzznowman.dyndns.org>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Request handlers

package ZIB::RequestHandlers;
use strict;
use warnings;
use vars qw($VERSION);
use ZIB qw(:logging :conversions);
use ZIB::Request;

use LWP::Simple;
use URI::Escape;
use HTML::Entities;

$VERSION = 20060604.1;


# Do a fresh start.
ZIB::Request->unregister_all_types();
ZIB::Request->emptyreplies(qw{:'( zilch nada nix nil null naught nought zero nothing});


# Request types: hello, zib
ZIB::Request->register_types(sub {
	my ($self) = @_;
	my $nick = raw2nick($self->who());

	$self->queue_reply("Hello $nick, I'm zib, your friendly bot.");
	$self->ready(1);
}, qw(hello zib));


# Use CrudeLeo if it is available.
eval { require CrudeLeo; CrudeLeo->import(); };

# Request types: dict, leo (using CrudeLeo)
ZIB::Request->register_types(sub {
	my ($self) = @_;
	my @args = $self->args();

	unless (defined(&crudeleo))
	{
		$self->queue_reply('CrudeLeo module not loaded');
		$self->ready(1);
		return;
	}

	my @results = crudeleo("@args", 5);
	my ($width1, $width2) = (1)x2;

	foreach my $pair (@results)
	{
		my ($len1, $len2) = map { length } (@$pair);

		$width1 = $len1 if ($len1 > $width1);
		$width2 = $len2 if ($len2 > $width2);
	}

	$width1++; $width2++;

	foreach my $pair (@results)
	{
		$self->queue_reply(
		  sprintf("%-${width1}s - %${width2}s", @$pair));
	}

	$self->ready(1);
}, qw(dict leo));



# Request types: wetter, weather
ZIB::Request->register_types(sub {
	my ($self) = @_;
	my $wetter = get('http://www.wetter.com/v2/?SID=&LANG=DE&LOC=7004&LOCFROM=0001&LOCFULL=7000&x=0&y=0&search='
	                 . uri_escape(join(' ', $self->args())));

	# TODO:
	#  - better error checking/handling
	#  - limit generated output to sane amount

	# check for ambiguity error
	if ($wetter =~ m#Ihre Suche nach .*? ergab \d+ Treffer(.*?)</ul>#s)
	{
		my $rest = $1;
		
		while ($rest =~ m#<li>(.*?)</li>#sg)
		{
			my $li = $1;

			$li =~ s/<.*?>//g;
			$li =~ s/^\s+|\s+$//g;

			$self->queue_reply(decode_entities($li))
			  if (length($li));
		}
	}
	
	# place
	return $self->ready(1)
	  unless ($wetter =~ m#\"TAB_BOX_2_HEADER\"[^>]*><span\s+class="HeadlineXL">(.*?)</span>(.*)#s);
	$wetter = $2;
	my $begin = 'Wetter fr: ' . decode_entities($1);

	# time
	return $self->ready(1)
	  unless ($wetter =~ m#\"TAB_BOX_2_HEADER\"[^>]*><span\s+class="Headline">(.*?)</span>(.*)#s);
	$wetter = $2;
	$self->queue_reply($begin . '  (' . decode_entities($1) . ')');

	# rest
	return $self->ready(1)
	  unless ($wetter =~ m#(.*?)<a\s+#s);
	my $rest = $1;
	undef $wetter;

	my @data;
	while ($rest =~ m#<tr>(.*?)</tr>#sg)
	{
		my $row = $1;
		my @group;

		while ($row =~ m#<span(?:\s[^>]+)?>(.*?)</span>#sg)
		{
			push(@group, decode_entities($1));
		}

		push(@data, join(' ', @group))
		  if (@group);
	}
	$self->queue_reply(join(' - ', @data));

	$self->ready(1);
}, qw(wetter weather));

1;
